@@ -133,7 +133,7 @@ def boxes_painter(rgb_image, box_list, label_list=None, score_list=None, color_l
             font = ImageFont.truetype('arial.ttf', fontsize)
         else:
             font = ImageFont.truetype(fm.findfont(fm.FontProperties(family='DejaVu Sans')),fontsize)
-    except IOError:
+    except:
         font = ImageFont.load_default()
 
     # draw boxes
@@ -251,4 +251,9 @@ def adjust_bbox(bbox, offset_list):
         bbox[3] + offset_list[3]
     ]
     
-    return new_box
\ No newline at end of file
+    return new_box
+   
+def xywh2xyxy(xywh):
+    assert len(xywh) == 4
+    bbox = [xywh[0], xywh[1], xywh[0] + xywh[2], xywh[1] + xywh[3]]
+    return bbox
