@@ -19,6 +19,7 @@ from ..utils import has_chinese_char
 from ..utils import vvd_floor
 from ..utils import change_file_name_for_path
 from ..utils import encode_path
+from ..utils import cal_distance
 
 from tqdm import tqdm
 from pathlib import Path
@@ -63,7 +64,7 @@ def get_bool_gravity_center(mask):
     gra_cen_y = np.sum(np.abs(mask).T * np.arange(mask.shape[0]) / max(1, num))
     return gra_cen_x, gra_cen_y
 
-def get_gravity_center(image, mask=None):
+def get_grey_level_gravity_center(image, mask=None):
     image = image.astype('uint8')
     if mask:
         img = image[mask!=0]
@@ -73,6 +74,22 @@ def get_gravity_center(image, mask=None):
     image_gravity_center = np.sum(value * np.arange(256)) / max(np.sum(value), 1)
     return image_gravity_center
 
+def gravity_center_xy(image):
+    bool_mask = to_gray_image(image) !=0
+
+    if not bool_mask.any():
+        return None, None
+
+    H, W = bool_mask.shape[:2]
+
+    X_array = np.arange(W)
+    Y_array = np.arange(H)
+
+    C_X = np.mean((bool_mask * X_array)[bool_mask])
+    C_Y = np.mean((bool_mask.T * Y_array)[bool_mask.T])
+
+    return C_X, C_Y
+
 def contour_center(mask):
     contours, hierarchy = cv2.findContours((mask != 0).astype('uint8'), 2, 1)
     points = np.vstack(contours).squeeze()
@@ -118,6 +135,9 @@ def center_crop(input_image, width, height):
 
 
 def image_resize(img_source, shape=None, factor=None, unique_check=False, interpolation=None):
+    if str(img_source.dtype).lower() != 'uint8':
+        print(f"Waring from mtutils image_resize: input source type {str(img_source.dtype)} is not uint8, we've made a transfer automatically.")
+        img_source = img_source.astype('uint8')
     image_H, image_W = img_source.shape[:2]
     if shape is not None:
         if is_number(shape):
@@ -330,6 +350,7 @@ def img_rotate(img,
     rotated.astype(type)
     return rotated
 
+image_rotate = img_rotate
 
 def image_flip_lr(image, contiguousarray=True):
     fliped = np.fliplr(image)
@@ -551,19 +572,23 @@ def image_show_from_path(file_path):
         plt_image_show(image)
 
 
-def plt_image_show(*image, window_name='', array_res=False, full_screen=True, cmap=None, position=[30, 30], share_xy=False, axis_off=False, col_num=None):
+def plt_image_show(*image, window_name='', array_res=False, full_screen=True, cmap=None, position=[30, 30], share_xy=False, axis_off=False, col_num=None, row_num=None, norm_float=True):
     '''
     更加鲁棒地显示图像包括二维图像,第三维度为1的图像
     '''
     image_list = list(image)
     # temp_image = extend_image_channel(image)
     image_num = len(image_list)
-    if col_num is None:
+    if col_num is None and row_num is None:
         col_num = int(np.ceil(image_num**0.5))
+    elif row_num is not None:
+        row_num = vvd_round(row_num)
+        assert row_num > 0
+        col_num = int(np.ceil(image_num/row_num))
     else:
         col_num = vvd_round(col_num)
         assert col_num > 0
-    row_num = int(np.ceil(image_num/col_num))
+        row_num = int(np.ceil(image_num/col_num))
 
     image_list = image_list + [None] * (col_num * row_num - image_num)
 
@@ -628,7 +653,10 @@ def plt_image_show(*image, window_name='', array_res=False, full_screen=True, cm
             elif 'bool' in image.dtype.__str__():
                 cur_ax.imshow(image.astype('float32'), cmap=cmap, vmax=np.max(image), vmin=np.min(image))
             elif 'float' in image.dtype.__str__():
-                cur_ax.imshow((image - np.min(image)) / (max(1, np.max(image)) - np.min(image)), cmap=cmap)
+                if norm_float:
+                    cur_ax.imshow((image - np.min(image)) / (max(1, np.max(image)) - np.min(image)), cmap=cmap)
+                else:
+                    cur_ax.imshow(image, cmap=cmap)
             else:
                 cur_ax.imshow(image.astype('uint8'), cmap=cmap, vmax=np.max(image), vmin=np.min(image))
 
@@ -979,11 +1007,18 @@ def rectangle2polygon(rec):
 
 
 def bbox2polygon(bbox):
+    if not bbox:
+        return []
     x1, y1, x2, y2 = bbox
     polygon = xyxy2polygon(x1, y1, x2, y2)
     return polygon
 
 
+def polygon2bbox(polygon):
+    array = np.array(polygon)
+    return np.min(array, axis=0).tolist() + np.max(array, axis=0).tolist()
+
+
 def xyxy2polygon(x1, y1, x2, y2):
     points = []
     points.append([x1, y1])
@@ -993,6 +1028,24 @@ def xyxy2polygon(x1, y1, x2, y2):
     return points
 
 
+def polygon_interpolation(polygon, max_distance, close=True):
+    assert max_distance > 0, f'distance {max_distance} should > 0'
+    if len(polygon) < 2:
+        return polygon
+    if len(polygon) > 2 and close:
+        polygon.append(polygon[0])
+
+    points_res = list()
+    for index in range(len(polygon)-1):
+        cur_point = polygon[index]
+        next_point = polygon[index+1]
+        assert len(cur_point) == len(next_point) == 2, 'point length should be 2'
+        interpolate_num = int(cal_distance(cur_point, next_point) // max_distance)
+        points_res.extend(np.linspace(cur_point, next_point, 2+interpolate_num).tolist()[:-1])
+    points_res.append(polygon[-1])
+    return points_res
+
+
 def get_polygons_from_mask(mask):
     """ get polygons for all isolated areas on fg-mask
         return something like: [[x1,y1], [x2,y2], [x3,y3], ...]
@@ -1002,6 +1055,24 @@ def get_polygons_from_mask(mask):
     return contours
 
 
+def get_shapes_from_mask(mask, epsilon=None, drop_small_polygon=False):
+    contours = get_polygons_from_mask(mask)
+    if epsilon is not None:
+        new_contours = list()
+        for contour in contours:
+            new_contour = cv2.approxPolyDP(contour, epsilon, True)
+            if new_contour.shape[0] > 2:
+                new_contours.append(new_contour)
+            else:
+                if not drop_small_polygon:
+                    new_contours.append(contour)
+        contours = new_contours
+    shapes = list()
+    for contour in contours:
+        shapes.append(contour[:, 0, :].tolist())
+    return shapes
+
+
 def draw_polygons(mask, polygons, color=None, thickness=5, fill=False):
     mask = mask.copy()
     if color is None:
@@ -1015,7 +1086,7 @@ def draw_polygons(mask, polygons, color=None, thickness=5, fill=False):
 
     return mask
 
-def draw_boxes(image, bboxes, color=None, thickness=5, fill=False):
+def draw_boxes(image, bboxes, color=None, thickness=1, fill=False):
     image = image.copy()
     if color is None:
         color = [255, 255, 255]
