@@ -46,7 +46,19 @@ from functools import reduce
 from func_timeout import func_set_timeout, FunctionTimedOut
 
 isfile = OS_isfile 
-exists = OS_exists
+
+
+def exists(input):
+    if isinstance(input, list):
+        res = True
+        for path in input:
+            if not OS_exists(str(path)):
+                res = False
+                break
+        return res
+    else:
+        return OS_exists(input)
+
 
 def get_list_from_list(data_list, call_back, absolutely=False):
     """[make a list through a input list,
@@ -196,6 +208,20 @@ def timer_vvd(func):
     return wrapper
 
 
+def two_way_dictionary(dict_obj):
+    length = len(dict_obj)
+    key_set = set(dict_obj.keys())
+    value_set = set(dict_obj.values())
+    
+    new_dict_obj = dict()
+    if len(set(value_set)) == length:
+        if len(value_set | key_set) == length * 2:
+            for key, value in dict_obj.items():
+                new_dict_obj[value] = key
+    new_dict_obj.update(dict_obj)
+    return new_dict_obj
+    
+
 def file_rename(srcFile, dstFile):
     """rename file
 
@@ -876,20 +902,23 @@ def whether_divisible_by(to_be_divided, dividing):
 def vvd_round(num):
     if iterable(num):
         return np.round(np.array(num)).astype('int32').tolist()
-    return int(round(num))
+    return int(np.round(num))
 
+round = vvd_round
 
 def vvd_ceil(num):
     if iterable(num):
         return np.ceil(np.array(num)).astype('int32').tolist()
     return int(np.ceil(num))
 
+ceil = vvd_ceil
 
 def vvd_floor(num):
     if iterable(num):
         return np.floor(np.array(num)).astype('int32').tolist()
     return int(np.floor(num))
 
+floor = vvd_floor
 
 def get_gpu_str_as_you_wish(gpu_num_wanted, verbose=0):
     """[get empty gpu index str as needed]
@@ -1022,10 +1051,23 @@ def try_exc_handler(try_func, exc_func, developer_mode=False):
             ori_exception_info = list(e.args)
             if len(ori_exception_info) == 0:
                 ori_exception_info.append('')
-            ori_exception_info[0] = ' ErrorMessage: ' + str(ori_exception_info[0])\
-                + '\n CrashFile: ' + str(e.__traceback__.tb_next.tb_frame.f_globals['__file__'])\
-                + '\n Line: ' + str(e.__traceback__.tb_next.tb_lineno)
+            exception_str = ' ErrorMessage: ' + str(ori_exception_info[0])
+            if hasattr(e, '__traceback__'):
+                cur_trace_obj = e.__traceback__
+                level = 1
+                while True:
+                    if not hasattr(cur_trace_obj, 'tb_next'):
+                        break
+                    if level > 300:
+                        break
+                    frame_str = str(cur_trace_obj.tb_frame)
+                    frame_str = frame_str[max(0, frame_str.find('file')):]
+                    res_str =  '\n' + str(level) + ': ' + frame_str
+                    exception_str += res_str
+                    cur_trace_obj = cur_trace_obj.tb_next
+                    level += 1
 
+            ori_exception_info[0] = exception_str
             e.args = tuple(ori_exception_info)
             except_result = exc_func(e)
         except Exception as ee:
@@ -1048,6 +1090,7 @@ def try_exc_handler(try_func, exc_func, developer_mode=False):
     return try_result
 
 
+
 def class_timer(input_class):
     """
     get a new class through a Decorator
